      program basic2dsc
c     translates Kevin Jardine's "basic_data.xls" (in csv format) 
c     into dsc and sript format

      implicit none
      character*255 line,linetmp
      character*32 type,name,mesh(0:4000)
      character*5 color(5)
      data color / ' ','0 1 0','0 0 1','1 0 0','1 1 1'/
      integer nmesh(0:4000),nline,nstars,notstars,nmods,nmod,i
      integer ncore, lun, lun_co,lun_cl,lun_hii,lun_st
      integer num_close,num_far,ntot,nl
      data lun_co,lun_cl,lun_hii,lun_st / 2,3,4,5/
      logical lcore
      data nmesh/4001*0/
      double precision long,lat,dist,diam,radius,l2,b2,ra,dec
      double precision size

      open (unit=1,file='milky-way-basic_data.csv',form='formatted',
      1 type='old')
      open (unit=2,file='milky-way-co.dscNO',form='formatted')
      open (unit=3,file='milky-way-cl.dscNO',form='formatted')
      open (unit=4,file='milky-way-hii.dscNO',form='formatted')
      open (unit=5,file='milky-way-st.dscNO',form='formatted')

      open (unit=12,file='milky-way-co.cel',form='formatted')
      open (unit=13,file='milky-way-cl.cel',form='formatted')
      open (unit=14,file='milky-way-hii.cel',form='formatted')
      open (unit=15,file='milky-way-st.cel',form='formatted')

      open (unit=22,file='milky-way-co.stc',form='formatted')
      open (unit=23,file='milky-way-cl.stc',form='formatted')
      open (unit=24,file='milky-way-hii.stc',form='formatted')
      open (unit=25,file='milky-way-st.stc',form='formatted')

      do nl = 12,15
         write(nl,*) '{ '
         enddo

c     skip column headers
      read (unit=1,err=20,end=30,fmt='(a)') line

      nline=0
      nstars=0
      notstars=0
      nmod=0
      ncore=0
      num_close=0
      num_far=0

 100  continue
      read (unit=1,err=20,end=30,fmt='(a)') line

      call get_info (line,type,long,lat,dist,name,radius)

c      print *, line,type,long,lat,dist,name,radius
      nline=nline+1

      l2=long
      b2=lat

c     test to see if object is near 0 l2
c     (excess objects in line toward galactic center:
c     it seemed that most of them were within 1/2 degree)
c     fixed.

      if (l2.lt.0.5 .or. l2.gt.359.5) then
c         print *,'toward core? ',type(:3),' ',name,' ',l2,b2,dist
         ncore=ncore+1
         lcore=.true.
         else
            lcore=.false.
         endif
    

      call gal2eq ( l2, b2, ra,dec)


      if (type(1:3).eq.'"co') lun = lun_co
      if (type(1:3).eq.'"cl') lun = lun_cl
      if (type(1:3).eq.'"hi') lun = lun_hii
      if (type(1:3).eq.'"st') lun = lun_st

      if (dist.gt.0.01) then
         write (unit=lun,fmt=*) 'Nebula ',name
         write (lun,*) '{ RA ',ra,' Dec ',dec
         write (lun,*) '  Distance ',dist,'  Radius ',radius
         write (lun,*) ' Mesh ',type,' }'

         size=radius/10.0
         if (size.lt.1.0)size=1.0

	write (lun+10,*)
        1 'mark { object ',name,'size',size,' color [',
        1 color(lun),'] symbol "circle" }'

	write (lun+20,*)
         write (lun+20,fmt=*) 'Star ',name
         write (lun+20,*) '{ RA ',ra*15.0,' Dec ',dec
         write (lun+20,*) '  Distance ',dist,'  Radius ',radius
         write (lun+20,*) ' Mesh ',type,' SpectralType "M9" AbsMag 20 }'

         num_far = num_far+1

      if (type(1:3).ne.'"st') then
         notstars=notstars+1
      else
         nstars=nstars+1
      endif

c     count and list different models

         do nmods=1,nmod
         if (mesh(nmods).eq.type) then
		nmesh(nmods)=nmesh(nmods)+1
		goto 12
		endif
         enddo

      nmod=nmod+1
      mesh(nmod)=type
      nmesh(nmod)=1
 12   continue

         else
            num_close=num_close+1
         endif

      goto 100

 20   continue
      print *, 'read error'
 30   continue
c==========================================
c     summary
      print *, nline,' lines read'
      print *, 'nstars=',nstars,', notstars=',notstars
      print *, ' tot=',nstars+notstars
      print *,' nmod=',nmod
      print 31, (nmesh(i),mesh(i),i=1,nmod)

      ntot=0
      do i=1,nmod
      ntot = ntot+nmesh(i)
      enddo
      print *, ' tot=',ntot
      print *, 'toward core: ',ncore
      print *, 'num_close=',num_close,',num_far=',num_far
      print *, ' tot=',num_close+num_far
 31   format(1x,i5,a)

            do nl = 2,5
         close(nl)
         write(nl+10,*) '}'
         close(nl+10)
         close(nl+20)
         enddo

      end
c==============================================================
      subroutine get_info (line,type,long,lat,dist,name,radius)

      character*255 line
      character*32 type,name
      double precision long,lat,dist,diam,radius
      character*32 field(9)
      integer ipc(0:9)

c     9 data fields
      do nc = 0,9
         ipc(nc)=0
         enddo

c     find commas
c     assume no commas in quoted strings

      ncom=0
      do nc = 1,255
         if (line(nc:nc).eq.',') then
            ncom=ncom+1
            ipc(ncom)=nc
         endif
       enddo

c     simulate final comma
       do nc = 255,1,-1
          if (line(nc:nc).ne.' ') then
             ipc(9)=nc+1
             goto 1
             endif
        enddo
 1      continue

      do nf = 1,9
c     extract individual fields
         field(nf) = line(ipc(nf-1)+1:ipc(nf)-1)
         enddo

c     make sure all numbers are floating point:
c     test for a dot and append one if it's missing

         do nf = 3,5
            call fix_float (field(nf))
         enddo
c     fix diameter too
            call fix_float (field(8))

c     add model suffix to type
         do nc=32,1,-1
            if (field(2)(nc:nc).ne.' ')then
               field(2)(nc:nc+5)='.cmod"'
               goto 11
            endif
          enddo
 11       continue

c     translate fields to binary

         read (unit=field(1),fmt='(i5)') id
         read (unit=field(2),fmt='(a)') type
         read (unit=field(3),fmt='(f9.3)') long
         read (unit=field(4),fmt='(f9.3)') lat
         read (unit=field(5),fmt='(f9.3)') dist
         read (unit=field(6),fmt='(a)') name
c     skip map sector
         read (unit=field(8),fmt='(f9.3)') diam

c         print *, line
c         print *, field

c     convert parsecs & diam to ly and radius
         dist = dist*3.26163
         radius = diam*3.26163/2.0
         if (radius.lt.0.1) radius = 50.1

c         print *, dist,radius
      return
      end
c==============================================================
       subroutine fix_float (field)
       character*32 field
            do nc = 1,32
               if (field(nc:nc).eq.'.') return
               enddo

c     convert integers to fp
         do nc=32,1,-1
            if (field(nc:nc).ne.' ')then
               field(nc+1:nc+2)='.0'
               return
            endif
          enddo

c     can't get here... yeah, right
         return
         end
c==============================================================
      subroutine gal2eq (l2, b2, ra,dec)
      implicit double precision (a-z)

      DOUBLE PRECISION D2PI
      PARAMETER (D2PI=6.283185307179586476925286766559D0)

      dl = d2pi*l2/360.0d0
      db = d2pi*b2/360.0d0
      CALL sla_GALEQ (DL, DB, DR, DD)
      ra = (dr*360.0d0/d2pi)/15.0d0
      dec = dd*360.0d0/d2pi
      return

      END
c==============================================================

      SUBROUTINE sla_GALEQ (DL, DB, DR, DD)
*+
*     - - - - - -
*      G A L E Q
*     - - - - - -
*
*  Transformation from IAU 1958 galactic coordinates to
*  J2000.0 equatorial coordinates (double precision)
*
*  Given:
*     DL,DB       dp       galactic longitude and latitude L2,B2
*
*  Returned:
*     DR,DD       dp       J2000.0 RA,Dec
*
*  (all arguments are radians)
*
*  Called:
*     sla_DCS2C, sla_DIMXV, sla_DCC2S, sla_DRANRM, sla_DRANGE
*
*  Note:
*     The equatorial coordinates are J2000.0.  Use the routine
*     sla_GE50 if conversion to B1950.0 'FK4' coordinates is
*     required.
*
*  Reference:
*     Blaauw et al, Mon.Not.R.Astron.Soc.,121,123 (1960)
*
*  P.T.Wallace   Starlink   21 September 1998
*
*  Copyright (C) 1998 Rutherford Appleton Laboratory
*-

      IMPLICIT NONE

      DOUBLE PRECISION DL,DB,DR,DD

      DOUBLE PRECISION sla_DRANRM,sla_DRANGE

      DOUBLE PRECISION V1(3),V2(3)

*
*  L2,B2 system of galactic coordinates
*
*  P = 192.25       RA of galactic north pole (mean B1950.0)
*  Q =  62.6        inclination of galactic to mean B1950.0 equator
*  R =  33          longitude of ascending node
*
*  P,Q,R are degrees
*
*  Equatorial to galactic rotation matrix (J2000.0), obtained by
*  applying the standard FK4 to FK5 transformation, for zero proper
*  motion in FK5, to the columns of the B1950 equatorial to
*  galactic rotation matrix:
*
      DOUBLE PRECISION RMAT(3,3)
      DATA RMAT(1,1),RMAT(1,2),RMAT(1,3),
     :     RMAT(2,1),RMAT(2,2),RMAT(2,3),
     :     RMAT(3,1),RMAT(3,2),RMAT(3,3)/
     : -0.054875539726D0,-0.873437108010D0,-0.483834985808D0,
     : +0.494109453312D0,-0.444829589425D0,+0.746982251810D0,
     : -0.867666135858D0,-0.198076386122D0,+0.455983795705D0/



*  Spherical to Cartesian
      CALL sla_DCS2C(DL,DB,V1)

*  Galactic to equatorial
      CALL sla_DIMXV(RMAT,V1,V2)

*  Cartesian to spherical
      CALL sla_DCC2S(V2,DR,DD)

*  Express in conventional ranges
      DR=sla_DRANRM(DR)
      DD=sla_DRANGE(DD)

      END
      SUBROUTINE sla_DIMXV (DM, VA, VB)
*+
*     - - - - - -
*      D I M X V
*     - - - - - -
*
*  Performs the 3-D backward unitary transformation:
*
*     vector VB = (inverse of matrix DM) * vector VA
*
*  (double precision)
*
*  (n.b.  the matrix must be unitary, as this routine assumes that
*   the inverse and transpose are identical)
*
*  Given:
*     DM       dp(3,3)    matrix
*     VA       dp(3)      vector
*
*  Returned:
*     VB       dp(3)      result vector
*
*  P.T.Wallace   Starlink   March 1986
*
*  Copyright (C) 1995 Rutherford Appleton Laboratory
*-

      IMPLICIT NONE

      DOUBLE PRECISION DM(3,3),VA(3),VB(3)

      INTEGER I,J
      DOUBLE PRECISION W,VW(3)



*  Inverse of matrix DM * vector VA -> vector VW
      DO J=1,3
         W=0D0
         DO I=1,3
            W=W+DM(I,J)*VA(I)
         END DO
         VW(J)=W
      END DO

*  Vector VW -> vector VB
      DO J=1,3
         VB(J)=VW(J)
      END DO

      END
      SUBROUTINE sla_DCC2S (V, A, B)
*+
*     - - - - - -
*      D C C 2 S
*     - - - - - -
*
*  Direction cosines to spherical coordinates (double precision)
*
*  Given:
*     V     d(3)   x,y,z vector
*
*  Returned:
*     A,B   d      spherical coordinates in radians
*
*  The spherical coordinates are longitude (+ve anticlockwise
*  looking from the +ve latitude pole) and latitude.  The
*  Cartesian coordinates are right handed, with the x axis
*  at zero longitude and latitude, and the z axis at the
*  +ve latitude pole.
*
*  If V is null, zero A and B are returned.
*  At either pole, zero A is returned.
*
*  P.T.Wallace   Starlink   July 1989
*
*  Copyright (C) 1995 Rutherford Appleton Laboratory
*-

      IMPLICIT NONE

      DOUBLE PRECISION V(3),A,B

      DOUBLE PRECISION X,Y,Z,R


      X = V(1)
      Y = V(2)
      Z = V(3)
      R = SQRT(X*X+Y*Y)

      IF (R.EQ.0D0) THEN
         A = 0D0
      ELSE
         A = ATAN2(Y,X)
      END IF

      IF (Z.EQ.0D0) THEN
         B = 0D0
      ELSE
         B = ATAN2(Z,R)
      END IF

      END
      DOUBLE PRECISION FUNCTION sla_DRANRM (ANGLE)
*+
*     - - - - - - -
*      D R A N R M
*     - - - - - - -
*
*  Normalize angle into range 0-2 pi  (double precision)
*
*  Given:
*     ANGLE     dp      the angle in radians
*
*  The result is ANGLE expressed in the range 0-2 pi (double
*  precision).
*
*  P.T.Wallace   Starlink   23 November 1995
*
*  Copyright (C) 1995 Rutherford Appleton Laboratory
*-

      IMPLICIT NONE

      DOUBLE PRECISION ANGLE

      DOUBLE PRECISION D2PI
      PARAMETER (D2PI=6.283185307179586476925286766559D0)


      sla_DRANRM=MOD(ANGLE,D2PI)
      IF (sla_DRANRM.LT.0D0) sla_DRANRM=sla_DRANRM+D2PI

      END
      DOUBLE PRECISION FUNCTION sla_DRANGE (ANGLE)
*+
*     - - - - - - -
*      D R A N G E
*     - - - - - - -
*
*  Normalize angle into range +/- pi  (double precision)
*
*  Given:
*     ANGLE     dp      the angle in radians
*
*  The result (double precision) is ANGLE expressed in the range +/- pi.
*
*  P.T.Wallace   Starlink   23 November 1995
*
*  Copyright (C) 1995 Rutherford Appleton Laboratory
*-

      IMPLICIT NONE

      DOUBLE PRECISION ANGLE

      DOUBLE PRECISION DPI,D2PI
      PARAMETER (DPI=3.141592653589793238462643D0)
      PARAMETER (D2PI=6.283185307179586476925287D0)


      sla_DRANGE=MOD(ANGLE,D2PI)
      IF (ABS(sla_DRANGE).GE.DPI)
     :          sla_DRANGE=sla_DRANGE-SIGN(D2PI,ANGLE)

      END
      SUBROUTINE sla_DCS2C (A, B, V)
*+
*     - - - - - -
*      D C S 2 C
*     - - - - - -
*
*  Spherical coordinates to direction cosines (double precision)
*
*  Given:
*     A,B       dp      spherical coordinates in radians
*                        (RA,Dec), (Long,Lat) etc
*
*  Returned:
*     V         dp(3)   x,y,z unit vector
*
*  The spherical coordinates are longitude (+ve anticlockwise
*  looking from the +ve latitude pole) and latitude.  The
*  Cartesian coordinates are right handed, with the x axis
*  at zero longitude and latitude, and the z axis at the
*  +ve latitude pole.
*
*  P.T.Wallace   Starlink   October 1984
*
*  Copyright (C) 1995 Rutherford Appleton Laboratory
*-

      IMPLICIT NONE

      DOUBLE PRECISION A,B,V(3)

      DOUBLE PRECISION COSB



      COSB=COS(B)

      V(1)=COS(A)*COSB
      V(2)=SIN(A)*COSB
      V(3)=SIN(B)

      END
